@@ -1,8 +1,13 @@
 use crate::iter::{Iter, IterMut};
 use crate::ParallelParam;
 use crate::{assert_in_bounds, assert_in_bounds_inclusive};
-use core::marker::PhantomData;
-use core::ops::{Range, RangeFrom, RangeFull, RangeInclusive, RangeTo};
+use alloc::vec::Vec;
+use core::{
+    cmp::Ordering,
+    hash::{Hash, Hasher},
+    marker::PhantomData,
+    ops::{Range, RangeFrom, RangeFull, RangeInclusive, RangeTo},
+};
 
 /// A immutable dynamically-sized view into a contiguous heterogeneous sequence.
 /// Contiguous here means that elements are laid out so that every element is
@@ -122,9 +127,8 @@ impl<'a, Param: ParallelParam> ParallelSlice<'a, Param> {
     /// Returns an iterator over the [`ParallelSlice`].
     pub fn iter(&self) -> Iter<'a, Param> {
         Iter {
-            base: Param::as_ptr(self.storage),
-            idx: 0,
-            len: self.len,
+            ptr: Param::as_ptr(self.storage),
+            remaining: self.len,
             _marker: PhantomData,
         }
     }
@@ -139,6 +143,23 @@ impl<'a, Param: ParallelParam> ParallelSlice<'a, Param> {
     }
 }
 
+impl<'s, 'r, Param> Hash for ParallelSlice<'s, Param>
+where
+    Param: ParallelParam + 's,
+    Param::Ref<'s>: Hash,
+    'r: 's,
+{
+    fn hash<H>(&self, hasher: &mut H)
+    where
+        H: Hasher,
+    {
+        self.len.hash(hasher);
+        for item in self.iter() {
+            item.hash(hasher);
+        }
+    }
+}
+
 /// A mutable dynamically-sized view into a contiguous heterogeneous sequence.
 /// Contiguous here means that elements are laid out so that every element is
 /// the same distance from its neighbors.
@@ -328,6 +349,122 @@ impl<'a, Param: ParallelParam> ParallelSliceMut<'a, Param> {
         Param::as_mut(Param::ptr_at(self.storage, index))
     }
 
+    /// Sorts the slice with a comparator function.
+    ///
+    /// This function will allocate `sizeof(usize) * self.len` bytes as an intermediate sorting
+    /// buffer.
+    ///
+    /// This defers to the `core` implemenation of [`slice::sort_by`], so any properties it
+    /// has will also hold for this function.
+    ///
+    /// [`slice::sort_by`]: https://doc.rust-lang.org/std/primitive.slice.html#method.sort_by
+    pub fn sort_by<F>(&mut self, f: F)
+    where
+        F: Fn(Param::Ref<'a>, Param::Ref<'a>) -> Ordering,
+    {
+        let base = Param::as_ptr(self.storage);
+        self.sort_via(|indices| {
+            indices.sort_by(|a, b| unsafe {
+                f(
+                    Param::as_ref(Param::add(base, *a)),
+                    Param::as_ref(Param::add(base, *b)),
+                )
+            });
+        });
+    }
+
+    /// Sorts the slice with a key extraction function.
+    ///
+    /// This function will allocate `sizeof(usize) * self.len` bytes as an intermediate sorting
+    /// buffer.
+    ///
+    /// This defers to the `core` implemenation of [`slice::sort_by_key`], so any properties it
+    /// has will also hold for this function.
+    ///
+    /// [`slice::sort_by`]: https://doc.rust-lang.org/std/primitive.slice.html#method.sort_by_key
+    pub fn sort_by_key<K, F>(&mut self, f: F)
+    where
+        F: Fn(Param::Ref<'a>) -> K,
+        K: Ord,
+    {
+        let base = Param::as_ptr(self.storage);
+        self.sort_via(|indices| {
+            indices.sort_by_key(|idx| unsafe { f(Param::as_ref(Param::add(base, *idx))) });
+        });
+    }
+
+    /// Sorts the slice with a comparator function, but might not preserve the order of equal
+    /// elements.
+    ///
+    /// This function will allocate `sizeof(usize) * self.len` bytes as an intermediate sorting
+    /// buffer.
+    ///
+    /// This defers to the `core` implemenation of [`slice::sort_unstable_by`], so any properties it
+    /// has will also hold for this function.
+    ///
+    /// [`slice::sort_unstable_by`]: https://doc.rust-lang.org/std/primitive.slice.html#method.sort_unstable_by
+    pub fn sort_unstable_by<F>(&mut self, f: F)
+    where
+        F: Fn(Param::Ref<'a>, Param::Ref<'a>) -> Ordering,
+    {
+        let base = Param::as_ptr(self.storage);
+        self.sort_via(|indices| {
+            indices.sort_unstable_by(|a, b| unsafe {
+                f(
+                    Param::as_ref(Param::add(base, *a)),
+                    Param::as_ref(Param::add(base, *b)),
+                )
+            });
+        });
+    }
+
+    /// Sorts the slice with a key extraction function, but might not preserve the order of equal
+    /// elements.
+    ///
+    /// This function will allocate `sizeof(usize) * self.len` bytes as an intermediate sorting
+    /// buffer.
+    ///
+    /// This defers to the `core` implemenation of [`slice::sort_unstable_by_key`], so any properties
+    /// it has will also hold this function.
+    ///
+    /// [`slice::sort_unstable_by_key`]: https://doc.rust-lang.org/std/primitive.slice.html#method.sort_unstable_by_key
+    pub fn sort_unstable_by_key<K, F>(&mut self, f: F)
+    where
+        F: Fn(Param::Ref<'a>) -> K,
+        K: Ord,
+    {
+        let base = Param::as_ptr(self.storage);
+        self.sort_via(|indices| {
+            indices.sort_unstable_by_key(|idx| unsafe { f(Param::as_ref(Param::add(base, *idx))) });
+        });
+    }
+
+    #[inline(always)]
+    fn sort_via<F>(&mut self, f: F)
+    where
+        F: FnOnce(&mut Vec<usize>),
+    {
+        if self.len < 2 {
+            return;
+        }
+
+        let mut indices: Vec<usize> = (0..self.len).collect();
+        f(&mut indices);
+
+        // SAFE: All of the indices used here are valid.
+        unsafe {
+            for src in 0..self.len {
+                let dst = *indices.get_unchecked(src);
+                if src == dst {
+                    continue;
+                }
+                self.swap_unchecked(src, dst);
+                // TODO: Use swap_unchecked here when stablized.
+                indices.swap(src, dst);
+            }
+        }
+    }
+
     /// Returns a raw pointer to the slice’s buffer.
     ///
     /// The caller must ensure that the slice outlives the pointer this function returns, or else it will end up pointing
@@ -382,9 +519,7 @@ impl<'a, Param: ParallelParam> ParallelSliceMut<'a, Param> {
     #[inline]
     pub unsafe fn swap_unchecked(&mut self, a: usize, b: usize) {
         let base = Param::as_ptr(self.storage);
-        let a_ptr = Param::add(base, a);
-        let b_ptr = Param::add(base, b);
-        Param::swap(a_ptr, b_ptr);
+        Param::swap(Param::add(base, a), Param::add(base, b));
     }
 
     /// Reverses the order of elements in the [`ParallelSliceMut`], in place.
@@ -425,9 +560,8 @@ impl<'a, Param: ParallelParam> ParallelSliceMut<'a, Param> {
     /// Returns an iterator over the [`ParallelSliceMut`].
     pub fn iter(&self) -> Iter<'a, Param> {
         Iter {
-            base: Param::as_ptr(self.storage),
-            idx: 0,
-            len: self.len,
+            ptr: Param::as_ptr(self.storage),
+            remaining: self.len,
             _marker: PhantomData,
         }
     }
@@ -435,9 +569,8 @@ impl<'a, Param: ParallelParam> ParallelSliceMut<'a, Param> {
     /// Returns an iterator that allows modifying each value.
     pub fn iter_mut(&mut self) -> IterMut<'a, Param> {
         IterMut {
-            base: self.as_mut_ptrs(),
-            idx: 0,
-            len: self.len,
+            ptr: self.as_mut_ptrs(),
+            remaining: self.len,
             _marker: PhantomData,
         }
     }
@@ -485,6 +618,23 @@ impl<'a, Param: ParallelParam> ParallelSliceMut<'a, Param> {
     }
 }
 
+impl<'s, 'r, Param> Hash for ParallelSliceMut<'s, Param>
+where
+    Param: ParallelParam + 's,
+    Param::Ref<'s>: Hash,
+    'r: 's,
+{
+    fn hash<H>(&self, hasher: &mut H)
+    where
+        H: Hasher,
+    {
+        self.len.hash(hasher);
+        for item in self.iter() {
+            item.hash(hasher);
+        }
+    }
+}
+
 pub trait ParallelSliceIndex<T> {
     type Output;
     fn get(self, slice: &T) -> Option<Self::Output>;
