@@ -1,9 +1,10 @@
-use crate::iter::IntoIter;
-use crate::{assert_in_bounds, out_of_bounds};
-use crate::{ParallelParam, ParallelSliceMut};
+use crate::{assert_in_bounds, iter::IntoIter, out_of_bounds, ParallelParam, ParallelSliceMut};
 use alloc::vec::Vec;
-use core::fmt::{Debug, Formatter};
-use core::ops::{Deref, DerefMut};
+use core::{
+    fmt::{Debug, Formatter},
+    hash::{Hash, Hasher},
+    ops::{Deref, DerefMut},
+};
 
 /// A contiguously growable heterogenous array type.
 ///
@@ -79,7 +80,7 @@ impl<Param: ParallelParam> ParallelVec<Param> {
         }
     }
 
-    unsafe fn drop_range(&mut self, start: usize, end: usize) {
+    pub(crate) unsafe fn drop_range(&mut self, start: usize, end: usize) {
         let base = Param::as_ptr(self.storage);
         for idx in start..end {
             Param::drop(Param::add(base, idx));
@@ -312,6 +313,19 @@ where
     }
 }
 
+impl<'a, Param: ParallelParam> Hash for ParallelVec<Param>
+where
+    Param: 'a,
+    Param::Ref<'a>: Hash,
+{
+    fn hash<H>(&self, hasher: &mut H)
+    where
+        H: Hasher,
+    {
+        self.deref().hash(hasher);
+    }
+}
+
 impl<Param: ParallelParam> FromIterator<Param> for ParallelVec<Param> {
     fn from_iter<T>(iter: T) -> Self
     where
@@ -331,7 +345,14 @@ impl<Param: ParallelParam> IntoIterator for ParallelVec<Param> {
     type Item = Param;
     type IntoIter = IntoIter<Param>;
     fn into_iter(self) -> Self::IntoIter {
-        IntoIter { vec: self, idx: 0 }
+        let iter = IntoIter {
+            storage: self.storage,
+            capacity: self.capacity,
+            len: self.len,
+            idx: 0,
+        };
+        core::mem::forget(self);
+        iter
     }
 }
 
